@@ -1,7 +1,7 @@
 /*
  * tclExtdInt.h
  *
- * Standard internal include file for Extended Tcl library..
+ * Standard internal include file for Extended Tcl.
  *-----------------------------------------------------------------------------
  * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
  *
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclExtdInt.h,v 5.6 1996/02/12 18:15:24 markd Exp $
+ * $Id: tclExtdInt.h,v 8.0 1996/11/21 00:23:58 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -20,87 +20,18 @@
 #define TCLEXTDINT_H
 
 #include "tclExtend.h"
-#include "tclXconf.h"
-#include "tclInt.h"
 
-#include <sys/param.h>
-
-#include <math.h>
-
-#ifdef NO_VALUES_H
-#    include <limits.h>
+#if defined(__WIN32__) || defined(_WIN32)
+#   include "tclXwinPort.h"
 #else
-#    include <values.h>
-#endif
-
-#ifndef MAXDOUBLE
-#    define MAXDOUBLE HUGE_VAL
-#endif
-
-#include <grp.h>
-
-/*
- * Included the tcl file tclUnixPort.h after other system files, as it checks
- * if certain things are defined.
- */
-#include "tclUnixPort.h"
-
-/*
- * Use the real functions, not the Tcl interface that hides signals.
- */
-#undef open
-#undef read
-#undef waitpid
-#undef write
-
-/*
- * Define O_ACCMODE if <fcntl.h> does not define it.
- */
-#ifndef O_ACCMODE
-#    define O_ACCMODE  (O_RDONLY|O_WRONLY|O_RDWR)
+#   include "tclXunixPort.h"
 #endif
 
 /*
- * Make sure we have both O_NONBLOCK and O_NDELAY defined.
+ * After above, since some of our port definitions affect what gets defined
+ * tclInt.h and the Tcl port file.
  */
-#ifndef O_NONBLOCK
-#   define O_NONBLOCK O_NDELAY
-#endif
-#ifndef O_NDELAY
-#   define O_NDELAY O_NONBLOCK
-#endif
-
-
-/*
- * If tclPort.h has already included time.h, don't include it again, some
- * systems don't #ifdef inside of the file.  Also, on some of these systems,
- * time.h is included by sys/time.h, so don't include it again if that is the
- * case.
- */
-#if !defined (NO_SYS_TIME_H) && defined(TIME_WITH_SYS_TIME)
-#    include <time.h>
-#endif
-
-#include <sys/times.h>
-
-/*
- * Make sure CLK_TCK is defined.
- */
-#ifndef CLK_TCK
-#    ifdef HZ
-#        define CLK_TCK HZ
-#    else
-#        define CLK_TCK 60
-#    endif
-#endif
-
-/*
- * These should be take from an include file, but it got to be such a mess
- * to get the include files right that they are here for good measure.
- */
-struct tm *gmtime ();
-struct tm *localtime ();
-
+#include "tclInt.h"
 
 /*
  * Get ranges of integers and longs.
@@ -172,6 +103,59 @@ typedef struct {
 #define TCLX_EVAL_FILE            2  /* Read and evaluate a file.          */
 #define TCLX_EVAL_ERR_HANDLER     4  /* Call error handler on error.       */
 
+/*
+ * Defines used by TclX_Get/SetChannelOption.  Defines name TCLX_COPT_ are the
+ * options and the others are the value
+ */
+#define TCLX_COPT_BLOCKING      1
+#define TCLX_MODE_BLOCKING      0
+#define TCLX_MODE_NONBLOCKING   1
+
+#define TCLX_COPT_BUFFERING     2
+#define TCLX_BUFFERING_FULL     0
+#define TCLX_BUFFERING_LINE     1
+#define TCLX_BUFFERING_NONE     2
+
+/*
+ * Two values are always returned for translation, one for the read side and
+ * one for the write.  They are returned masked into one word.
+ */
+
+#define TCLX_COPT_TRANSLATION      3
+#define TCLX_TRANSLATE_READ_SHIFT  8
+#define TCLX_TRANSLATE_READ_MASK   0xFF00
+#define TCLX_TRANSLATE_WRITE_MASK  0x00FF
+
+#define TCLX_TRANSLATE_UNSPECIFIED 0 /* For only one direction specified */
+#define TCLX_TRANSLATE_AUTO        1
+#define TCLX_TRANSLATE_LF          2
+#define TCLX_TRANSLATE_BINARY      2  /* same as LF */
+#define TCLX_TRANSLATE_CR          3
+#define TCLX_TRANSLATE_CRLF        4
+#define TCLX_TRANSLATE_PLATFORM    5
+
+/*
+ * Flags used by chown/chgrp.
+ */
+#define TCLX_CHOWN  0x1
+#define TCLX_CHGRP  0x2
+
+/*
+ * Structure use to pass file locking information.  Parallels the Posix
+ * struct flock, but use to pass info from the generic code to the system
+ * dependent code.
+ */
+typedef struct {
+    Tcl_Channel channel;      /* Channel to lock */
+    int         access;       /* TCL_READABLE and/or TCL_WRITABLE */
+    int         block;        /* Block if lock is not available */
+    off_t       start;        /* Starting offset */
+    off_t       len;          /* len = 0 means until end of file */
+    pid_t       pid;          /* Lock owner */
+    short       whence;       /* Type of start */
+    int         gotLock;      /* Succeeded? */
+} TclX_FlockInfo;
+
 /*
  * Used to return argument messages by most commands.
  */
@@ -200,20 +184,19 @@ extern char *tclXWrongArgs;
 #define ckstrdup(sourceStr) \
   (strcpy (ckalloc (strlen (sourceStr) + 1), sourceStr))
 
-
+/*
+ * Callback type for walking directories.
+ */
+typedef int
+(TclX_WalkDirProc) _ANSI_ARGS_((Tcl_Interp  *interp,
+                                char        *path,
+                                char        *fileName,
+                                int          caseSensitive,
+                                ClientData   clientData));
 
 /*
  * Prototypes for utility procedures.
  */
-extern void
-Tcl_CloseForError _ANSI_ARGS_((Tcl_Interp *interp,
-                               Tcl_Channel channel,
-                               int         fileNum));
-
-extern int
-Tcl_CommandLoop _ANSI_ARGS_((Tcl_Interp *interp,
-                             int         interactive));
-
 extern int
 Tcl_StrToOffset _ANSI_ARGS_((CONST char *string,
                              int         base,
@@ -227,16 +210,7 @@ TclX_Eval _ANSI_ARGS_((Tcl_Interp  *interp,
 extern int
 TclX_VarEval _ANSI_ARGS_(TCL_VARARGS(Tcl_Interp *, arg1));
 
-Tcl_Channel
-TclX_Stdin _ANSI_ARGS_((Tcl_Interp  *interp));
-
-Tcl_Channel
-TclX_Stdout _ANSI_ARGS_((Tcl_Interp  *interp));
-
-Tcl_Channel
-TclX_Stderr _ANSI_ARGS_((Tcl_Interp  *interp));
-
-int
+extern int
 TclX_WriteStr _ANSI_ARGS_((Tcl_Channel  channel,
                            char        *str));
 
@@ -244,29 +218,19 @@ TclX_WriteStr _ANSI_ARGS_((Tcl_Channel  channel,
 
 
 extern int
-TclX_ChannelFnum _ANSI_ARGS_((Tcl_Channel channel,
-                              int         direction));
+TclX_GetChannelOption _ANSI_ARGS_((Tcl_Channel channel,
+                                   int         option));
 
 extern Tcl_Channel
 TclX_GetOpenChannel _ANSI_ARGS_((Tcl_Interp *interp,
                                  char       *handle,
-                                 int         accessMode));
-
-int
-TclX_GetOpenFnum _ANSI_ARGS_ ((Tcl_Interp *interp,
-                               char       *handle,
-                               int         accessMode));
+                                 int         direction));
 
 extern int
 Tcl_GetOffset _ANSI_ARGS_((Tcl_Interp *interp,
                            CONST char *string,
                            off_t      *offsetPtr));
 
-extern int
-Tcl_ProcessSignal _ANSI_ARGS_((ClientData  clientData,
-                               Tcl_Interp *interp,
-                               int         cmdResultCode));
-
 extern void
 TclX_RegExpClean _ANSI_ARGS_((TclX_regexp *regExpPtr));
 
@@ -290,17 +254,22 @@ Tcl_RelativeExpr _ANSI_ARGS_((Tcl_Interp  *interp,
                               long         stringLen,
                               long        *exprResultPtr));
 
-extern void
-Tcl_ResetSignals ();
+extern int
+TclXRuntimeInit _ANSI_ARGS_((Tcl_Interp *interp,
+                             char       *which,
+                             char       *defaultLib,
+                             char       *version));
 
-extern Tcl_Channel
-TclX_SetupFileEntry _ANSI_ARGS_((Tcl_Interp *interp,
-                                 int         fileNum,
-                                 int         mode,
-                                 int         isSocket));
+extern int
+TclX_SetChannelOption _ANSI_ARGS_((Tcl_Interp  *interp,
+                                   Tcl_Channel  channel,
+                                   int          option,
+                                   int          value));
 
-extern clock_t
-Tcl_TicksToMS _ANSI_ARGS_((clock_t numTicks));
+extern char *
+TclX_JoinPath _ANSI_ARGS_((char        *path1,
+                           char        *path2,
+                           Tcl_DString *joinedPath));
 
 /*
  * Definitions required to initialize all extended commands.  These are either
@@ -366,9 +335,6 @@ Tcl_CopyfileCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 extern int 
 Tcl_LgetsCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
-extern int 
-Tcl_FrenameCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
 extern int
 Tcl_FtruncateCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
@@ -457,6 +423,9 @@ Tcl_LassignCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 extern int 
 Tcl_LmatchCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
+extern int 
+Tcl_LcontainCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+
 /*
  * from tclXmath.c
  */
@@ -508,6 +477,9 @@ Tcl_CindexCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 extern int 
 Tcl_ClengthCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
+extern int
+Tcl_CconcatCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+
 extern int 
 Tcl_CrangeCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
@@ -536,13 +508,13 @@ extern void
 Tcl_InitLibrary _ANSI_ARGS_((Tcl_Interp *interp));
 
 /*
- * from tclXunixcmds.c
+ * from tclXoscmds.c
  */
 extern int 
 Tcl_AlarmCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
 extern int 
-Tcl_ChrootCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+Tcl_LinkCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
 extern int 
 Tcl_NiceCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
@@ -556,28 +528,247 @@ Tcl_SyncCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 extern int 
 Tcl_SystemCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
-extern int 
-Tcl_TimesCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
 extern int 
 Tcl_UmaskCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
+/*
+ * from tclXunixCmds.c or tclXwinCmds.c
+ */
 extern int 
-Tcl_LinkCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+Tcl_ChrootCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
 extern int 
-Tcl_UnlinkCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+Tcl_TimesCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
-extern int 
-Tcl_MkdirCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_RmdirCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
 /*
- * from tclXserver.c
+ * from tclXsocket.c
+ */
+extern int
+TclXGetHostInfo _ANSI_ARGS_((Tcl_Interp *interp,
+                             Tcl_Channel channel,
+                             int         remoteHost));
+
+extern int
+Tcl_HostInfoCmd _ANSI_ARGS_((ClientData  clientData,
+                             Tcl_Interp *interp,
+                             int         argc,
+                             char      **argv));
+
+/*
+ * from tclXunixSock.c and stubbed in tclXwinCmds.c.
  */
 extern void
-Tcl_ServerInit _ANSI_ARGS_((Tcl_Interp*));
+TclX_ServerInit _ANSI_ARGS_((Tcl_Interp *interp));
 
+/*
+ * From TclXxxxDup.c
+ */
+Tcl_Channel
+TclXOSDupChannel _ANSI_ARGS_((Tcl_Interp *interp,
+                              Tcl_Channel srcChannel,
+                              int         mode,
+                              char       *targetChannelId));
+
+Tcl_Channel
+TclXOSBindOpenFile _ANSI_ARGS_((Tcl_Interp *interp,
+                                char       *fileNumStr));
+
+/*
+ * from tclXxxxOS.c
+ */
+extern int
+TclXNotAvailableError _ANSI_ARGS_((Tcl_Interp *interp,
+                                   char       *funcName));
+
+extern clock_t
+TclXOSTicksToMS _ANSI_ARGS_((clock_t numTicks));
+
+extern int
+TclXOSgetpriority _ANSI_ARGS_((Tcl_Interp *interp,
+                               int        *priority,
+                               char       *funcName));
+
+extern int
+TclXOSincrpriority _ANSI_ARGS_((Tcl_Interp *interp,
+                                int         priorityIncr,
+                                int        *priority,
+                                char       *funcName));
+
+extern int
+TclXOSpipe _ANSI_ARGS_((Tcl_Interp *interp,
+                        Tcl_Channel *channels));
+
+extern int
+TclXOSsetitimer _ANSI_ARGS_((Tcl_Interp *interp,
+                             double     *seconds,
+                             char       *funcName));
+
+extern void
+TclXOSsleep _ANSI_ARGS_((unsigned seconds));
+
+extern void
+TclXOSsync _ANSI_ARGS_((void));
+
+extern int
+TclXOSfsync _ANSI_ARGS_((Tcl_Interp *interp,
+                         Tcl_Channel channel));
+
+extern int
+TclXOSsystem _ANSI_ARGS_((Tcl_Interp *interp,
+                          char       *command,
+                          int        *exitCode));
+
+extern int
+TclX_OSlink _ANSI_ARGS_((Tcl_Interp *interp,
+                         char       *srcPath,
+                         char       *destPath,
+                         char       *funcName));
+extern int
+TclX_OSsymlink _ANSI_ARGS_((Tcl_Interp *interp,
+                            char       *srcPath,
+                            char       *destPath,
+                            char       *funcName));
+
+extern void
+TclXOSElapsedTime _ANSI_ARGS_((clock_t *realTime,
+                               clock_t *cpuTime));
+
+extern int
+TclXOSkill _ANSI_ARGS_((Tcl_Interp *interp,
+                        pid_t       pid,
+                        int         signal,
+                        char       *funcName));
+
+extern int
+TclXOSFstat _ANSI_ARGS_((Tcl_Interp  *interp,
+                         Tcl_Channel  channel,
+                         int          direction,
+                         struct stat *statBuf,
+                         int         *ttyDev));
+    
+extern int
+TclXOSSeekable _ANSI_ARGS_((Tcl_Interp  *interp,
+                            Tcl_Channel  channel,
+                            int         *seekablePtr));
+
+extern int
+TclXOSWalkDir _ANSI_ARGS_((Tcl_Interp       *interp,
+                           char             *path,
+                           int               hidden,
+                           TclX_WalkDirProc *callback,
+                           ClientData        clientData));
+
+extern int
+TclXOSGetFileSize _ANSI_ARGS_((Tcl_Channel  channel,
+                               int          direction,
+                               off_t       *fileSize));
+
+extern int
+TclXOSftruncate _ANSI_ARGS_((Tcl_Interp  *interp,
+                             Tcl_Channel  channel,
+                             off_t        newSize,
+                             char        *funcName));
+
+extern int
+TclXOSfork _ANSI_ARGS_((Tcl_Interp *interp,
+                        char       *funcName));
+
+extern int
+TclXOSexecl _ANSI_ARGS_((Tcl_Interp *interp,
+                         char       *path,
+                         char      **argList));
+
+extern int
+TclXOSInetAtoN _ANSI_ARGS_((Tcl_Interp     *interp,
+                            char           *strAddress,
+                            struct in_addr *inAddress));
+
+extern int
+TclXOSgetpeername _ANSI_ARGS_((Tcl_Interp *interp,
+                               Tcl_Channel channel,
+                               void       *sockaddr,
+                               int         sockaddrSize));
+
+extern int
+TclXOSgetsockname _ANSI_ARGS_((Tcl_Interp *interp,
+                               Tcl_Channel channel,
+                               void       *sockaddr,
+                               int         sockaddrSize));
+
+extern int
+TclXOSgetsockopt _ANSI_ARGS_((Tcl_Interp  *interp,
+                              Tcl_Channel  channel,
+                              int          option,
+                              int         *valuePtr));
+
+extern int
+TclXOSsetsockopt _ANSI_ARGS_((Tcl_Interp  *interp,
+                              Tcl_Channel  channel,
+                              int          option,
+                              int          value));
+
+extern int
+TclXOSchmod _ANSI_ARGS_((Tcl_Interp *interp,
+                         char       *fileName,
+                         int         mode));
+
+extern int
+TclXOSfchmod _ANSI_ARGS_((Tcl_Interp *interp,
+                          Tcl_Channel channel,
+                          int         mode,
+                          char       *funcName));
+
+extern int
+TclXOSChangeOwnGrp _ANSI_ARGS_((Tcl_Interp  *interp,
+                                unsigned     options,
+                                char        *ownerStr,
+                                char        *groupStr,
+                                char       **files,
+                                char       *funcName));
+
+extern int
+TclXOSFChangeOwnGrp _ANSI_ARGS_((Tcl_Interp *interp,
+                                 unsigned    options,
+                                 char       *ownerStr,
+                                 char       *groupStr,
+                                 char      **channelIds,
+                                 char       *funcName));
+
+int
+TclXOSGetSelectFnum _ANSI_ARGS_((Tcl_Interp *interp,
+                                 Tcl_Channel channel,
+                                 int        *readFnumPtr,
+                                 int        *writeFnumPtr));
+
+int
+TclXOSHaveFlock _ANSI_ARGS_((void));
+
+int
+TclXOSFlock _ANSI_ARGS_((Tcl_Interp     *interp,
+                         TclX_FlockInfo *lockInfoPtr));
+
+int
+TclXOSFunlock _ANSI_ARGS_((Tcl_Interp     *interp,
+                           TclX_FlockInfo *lockInfoPtr));
+
+int
+TclXOSGetAppend _ANSI_ARGS_((Tcl_Interp *interp,
+                             Tcl_Channel channel,
+                             int        *valuePtr));
+
+int
+TclXOSSetAppend _ANSI_ARGS_((Tcl_Interp *interp,
+                             Tcl_Channel channel,
+                             int         value));
+
+int
+TclXOSGetCloseOnExec _ANSI_ARGS_((Tcl_Interp *interp,
+                                  Tcl_Channel channel,
+                                  int        *valuePtr));
+
+int
+TclXOSSetCloseOnExec _ANSI_ARGS_((Tcl_Interp *interp,
+                                  Tcl_Channel channel,
+                                  int         value));
 #endif
