@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXlib.c,v 5.2 1996/02/12 18:15:59 markd Exp $
+ * $Id: tclXlib.c,v 8.0 1996/11/21 00:24:11 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -49,8 +49,22 @@ static char *AUTO_PKG_INDEX = "auto_pkg_index";
  * This is a global rather than a local so it will work with K&R compilers.
  * Its writable so it works with gcc.
  */
-static char loadOusterCmd [] = "source $tclx_library/loadouster.tcl";
+static char loadOusterCmd [] = "source [file join $tclx_library loadouster.tcl]";
 
+/*
+ * Indicates the type of library index.
+ */
+typedef enum {
+    TCLLIB_TNDX,       /* *.tndx                   */
+    TCLLIB_TND,        /* *.tnd (.tndx in 8.3 land */
+    TCLLIB_OUSTER      /* tclIndex                 */
+} indexNameClass_t;
+
+/*
+ * Indicates if we have ITcl namespaces and the command to check for.
+ */
+static int haveNameSpaces = FALSE;
+static char nameSpaceCheckCommand [] = "@scope";
 
 /*
  * Per-interpreter structure used for managing the library.
@@ -67,7 +81,7 @@ static int
 EvalFilePart _ANSI_ARGS_((Tcl_Interp  *interp,
                           char        *fileName,
                           off_t        offset,
-                          unsigned     length));
+                          off_t        length));
 
 static char *
 MakeAbsFile _ANSI_ARGS_((Tcl_Interp  *interp,
@@ -107,13 +121,21 @@ BuildPackageIndex  _ANSI_ARGS_((Tcl_Interp *interp,
                                 char       *tlibFilePath));
 
 static int
-LoadPackageIndex _ANSI_ARGS_((Tcl_Interp *interp,
-                              char       *tlibFilePath));
+LoadPackageIndex _ANSI_ARGS_((Tcl_Interp       *interp,
+                              char             *tlibFilePath,
+                              indexNameClass_t  indexNameClass));
 
 static int
 LoadOusterIndex _ANSI_ARGS_((Tcl_Interp *interp,
                              char       *indexFilePath));
 
+static int
+LoadDirIndexCallback _ANSI_ARGS_((Tcl_Interp  *interp,
+                                  char        *path,
+                                  char        *fileName,
+                                  int          caseSensitive,
+                                  ClientData   clientData));
+
 static int
 LoadDirIndexes _ANSI_ARGS_((Tcl_Interp  *interp,
                             char        *dirName));
@@ -123,6 +145,12 @@ LoadPackageIndexes _ANSI_ARGS_((Tcl_Interp  *interp,
                                 libInfo_t   *infoPtr,
                                 char        *path));
 
+static int
+Tcl_Auto_load_pkgCmd _ANSI_ARGS_((ClientData   dummy,
+                                  Tcl_Interp  *interp,
+                                  int          argc,
+                                  char       **argv));
+
 static int
 AddInProgress _ANSI_ARGS_((Tcl_Interp  *interp,
                            libInfo_t   *infoPtr,
@@ -143,16 +171,36 @@ static int
 LoadAutoPath _ANSI_ARGS_((Tcl_Interp  *interp,
                           libInfo_t   *infoPtr));
 
+static int
+LoadStdProc _ANSI_ARGS_((Tcl_Interp  *interp,
+                         char        *command));
+
+static int
+LoadITclImportProc _ANSI_ARGS_((Tcl_Interp  *interp,
+                                char        *command));
+
 static int
 LoadCommand _ANSI_ARGS_((Tcl_Interp  *interp,
                          char        *command));
 
+static int
+Tcl_LoadlibindexCmd _ANSI_ARGS_((ClientData   dummy,
+                                 Tcl_Interp  *interp,
+                                 int          argc,
+                                 char       **argv));
+
+static int
+Tcl_Auto_loadCmd _ANSI_ARGS_((ClientData   clientData,
+                              Tcl_Interp  *interp,
+                              int          argc,
+                              char       **argv));
+
 static void
 TclLibCleanUp _ANSI_ARGS_((ClientData  clientData,
                            Tcl_Interp *interp));
+
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * EvalFilePart --
  *
  *   Read in a byte range of a file and evaulate it.
@@ -161,7 +209,7 @@ TclLibCleanUp _ANSI_ARGS_((ClientData  clientData,
  *   o interp (I) - A pointer to the interpreter, error returned in result.
  *   o fileName (I) - The file to evaulate.
  *   o offset (I) - Byte offset into the file of the area to evaluate
- *   o length (I) - Number of bytes to evaulate..
+ *   o length (I) - Number of bytes to evaulate.
  *-----------------------------------------------------------------------------
  */
 static int
@@ -169,61 +217,62 @@ EvalFilePart (interp, fileName, offset, length)
     Tcl_Interp  *interp;
     char        *fileName;
     off_t        offset;
-    unsigned     length;
+    off_t        length;
 {
-    Interp       *iPtr = (Interp *) interp;
-    int           fileNum, result;
-    struct stat   statBuf;
-    char         *oldScriptFile, *cmdBuffer, *buf;
-    Tcl_DString   tildeBuf;
+    Interp *iPtr = (Interp *) interp;
+    int result;
+    off_t fileSize;
+    Tcl_DString pathBuf, cmdBuf;
+    char *oldScriptFile, *buf;
+    Tcl_Channel channel = NULL;
 
     Tcl_ResetResult (interp);
-    Tcl_DStringInit (&tildeBuf);
-    
-    if (fileName [0] == '~') {
-        if ((fileName = Tcl_TranslateFileName (interp, fileName,
-                                               &tildeBuf)) == NULL)
-            return TCL_ERROR;
-    }
+    Tcl_DStringInit (&pathBuf);
+    Tcl_DStringInit (&cmdBuf);
 
-    fileNum = open (fileName, O_RDONLY, 0);
-    if (fileNum < 0) {
-        Tcl_AppendResult (interp, "open failed on: ", fileName, ": ",
-                          Tcl_PosixError (interp), (char *) NULL);
-        return TCL_ERROR;
-    }
+    fileName = Tcl_TranslateFileName (interp, fileName, &pathBuf);
+    if (fileName == NULL)
+        goto errorExit;
 
-    if (fstat (fileNum, &statBuf) == -1)
-        goto accessError;
+    channel = Tcl_OpenFileChannel (interp, fileName, "r", 0);
+    if (channel == NULL)
+        goto errorExit;
 
-    if ((statBuf.st_size < offset + length) || (offset < 0)) {
-        Tcl_AppendResult (interp, "range to eval outside of file bounds \"",
-                          fileName, "\"", (char *) NULL);
+    if (TclXOSGetFileSize (channel, TCL_READABLE, &fileSize) == TCL_ERROR)
+        goto posixError;
+
+    if ((fileSize < offset + length) || (offset < 0)) {
+        Tcl_AppendResult (interp, "range to eval outside of file bounds in \"",
+                          fileName, "\", index file probably corrupt",
+                          (char *) NULL);
         goto errorExit;
     }
-    if (lseek (fileNum, offset, 0) < 0)
-        goto accessError;
 
-    cmdBuffer = ckalloc (length + 1);
-    if (read (fileNum, cmdBuffer, length) != length)
-        goto accessError;
+    if (Tcl_Seek (channel, offset, SEEK_SET) < 0)
+        goto posixError;
 
-    cmdBuffer [length] = '\0';
+    Tcl_DStringSetLength (&cmdBuf, length + 1);
+    if (Tcl_Read (channel, cmdBuf.string, length) != length) {
+        if (Tcl_Eof (channel))
+            goto prematureEof;
+        else
+            goto posixError;
+    }
+    cmdBuf.string [length] = '\0';
 
-    if (close (fileNum) != 0)
-        goto accessError;
-    fileNum = -1;
+    if (Tcl_Close (NULL, channel) != 0)
+        goto posixError;
+    channel = NULL;
 
     oldScriptFile = iPtr->scriptFile;
     iPtr->scriptFile = fileName;
-
-    result = Tcl_GlobalEval (interp, cmdBuffer);
-
+    result = Tcl_GlobalEval (interp, cmdBuf.string);
     iPtr->scriptFile = oldScriptFile;
-    ckfree (cmdBuffer);
-                         
+    
+    Tcl_DStringFree (&pathBuf);
+    Tcl_DStringFree (&cmdBuf);
+
     if (result != TCL_ERROR) {
-        Tcl_DStringFree (&tildeBuf);
         return TCL_OK;
     }
 
@@ -241,28 +290,34 @@ EvalFilePart (interp, fileName, offset, length)
     /*
      * Errors accessing the file once its opened are handled here.
      */
-  accessError:
+  posixError:
     Tcl_AppendResult (interp, "error accessing: ", fileName, ": ",
                       Tcl_PosixError (interp), (char *) NULL);
+    goto errorExit;
+
+  prematureEof:
+    Tcl_AppendResult (interp, "premature EOF on: ", fileName, (char *) NULL);
+    goto errorExit;
 
   errorExit:
-    if (fileNum > 0)
-        close (fileNum);
-    Tcl_DStringFree (&tildeBuf);
+    if (channel != NULL)
+        Tcl_Close (NULL, channel);
+    Tcl_DStringFree (&pathBuf);
+    Tcl_DStringFree (&cmdBuf);
     return TCL_ERROR;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * MakeAbsFile --
  *
- * Convert a file name to an absolute path.  This handles tilde substitution
+ * Convert a file name to an absolute path.  This handles file name translation
  * and preappend the current directory name if the path is relative.
  *
  * Parameters
  *   o interp (I) - A pointer to the interpreter, error returned in result.
  *   o fileName (I) - File name (should not start with a "/").
- *   o absNamePtr (O) - The name is returned in this dynamic string.
+ *   o absNamePtr (O) - The name is returned in this dynamic string.  It
+ *     should be initialized.
  * Returns:
  *   A pointer to the file name in the dynamic string or NULL if an error
  * occured.
@@ -274,52 +329,47 @@ MakeAbsFile (interp, fileName, absNamePtr)
     char        *fileName;
     Tcl_DString *absNamePtr;
 {
-    char  curDir [MAXPATHLEN+1];
+    char  *curDir;
+    Tcl_DString joinBuf;
 
-    Tcl_DStringFree (absNamePtr);
+    Tcl_DStringSetLength (absNamePtr, 1);
+
+    fileName = Tcl_TranslateFileName (interp, fileName, absNamePtr);
+    if (fileName == NULL)
+        goto errorExit;
 
     /*
-     * If its already absolute, just copy the name.
+     * If its already absolute.  If name translation didn't actually
+     * copy the name to the buffer, we must do it now.
      */
-    if (fileName [0] == '/') {
-        Tcl_DStringAppend (absNamePtr, fileName, -1);
-        return Tcl_DStringValue (absNamePtr);
-    }
-
-    /*
-     * If it starts with a tilde, the substitution will make it
-     * absolute.
-     */
-    if (fileName [0] == '~') {
-        if (Tcl_TranslateFileName (interp, fileName, absNamePtr) == NULL)
-            return NULL;
+    if (Tcl_GetPathType (fileName) == TCL_PATH_ABSOLUTE) {
+        if (fileName != absNamePtr->string) {
+            Tcl_DStringAppend (absNamePtr, fileName, -1);
+        }
         return Tcl_DStringValue (absNamePtr);
     }
 
     /*
      * Otherwise its relative to the current directory, get the directory
-     * and go from here.
+     * and join into a path.
      */
-#ifdef HAVE_GETCWD
-    if (getcwd (curDir, MAXPATHLEN) == NULL) {
-        Tcl_AppendResult (interp, "error getting working directory name: ",
-                          Tcl_PosixError (interp), (char *) NULL);
-    }
-#else
-    if (getwd (curDir) == NULL) {
-        Tcl_AppendResult (interp, "error getting working directory name: ",
-                          curDir, (char *) NULL);
-    }
-#endif
-    Tcl_DStringAppend (absNamePtr, curDir,   -1);
-    Tcl_DStringAppend (absNamePtr, "/",      -1);
-    Tcl_DStringAppend (absNamePtr, fileName, -1);
+    curDir = TclGetCwd (interp);
+    if (curDir == NULL)
+        goto errorExit;
+
+    Tcl_DStringInit (&joinBuf);
+    TclX_JoinPath (curDir, fileName, &joinBuf);
+    Tcl_DStringSetLength (absNamePtr, 0);
+    Tcl_DStringAppend (absNamePtr, joinBuf.string, -1);
+    Tcl_DStringFree (&joinBuf);
 
     return Tcl_DStringValue (absNamePtr);
+
+  errorExit:
+    return NULL;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * SetPackageIndexEntry --
  *
  * Set a package entry in the auto_pkg_index array in the form:
@@ -363,8 +413,7 @@ SetPackageIndexEntry (interp, packageName, fileName, offset, length)
     return (setResult == NULL) ? TCL_ERROR : TCL_OK;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * GetPackageIndexEntry --
  *
  * Get a package entry from the auto_pkg_index array.
@@ -440,8 +489,7 @@ GetPackageIndexEntry (interp, packageName, fileNamePtr, offsetPtr, lengthPtr)
     return TCL_ERROR;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * SetProcIndexEntry --
  *
  * Set the proc entry in the auto_index array.  These entry contains a command
@@ -465,9 +513,8 @@ SetProcIndexEntry (interp, procName, package)
     char        *result;
 
     Tcl_DStringInit (&command);
-    Tcl_DStringAppend (&command, "auto_load_pkg {", -1);
-    Tcl_DStringAppend (&command, package, -1);
-    Tcl_DStringAppend (&command, "}", -1);
+    Tcl_DStringAppendElement (&command, "auto_load_pkg");
+    Tcl_DStringAppendElement (&command, package);
 
     result = Tcl_SetVar2 (interp, AUTO_INDEX, procName, command.string,
                           TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);
@@ -477,8 +524,7 @@ SetProcIndexEntry (interp, procName, package)
     return (result == NULL) ? TCL_ERROR : TCL_OK;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * AddLibIndexErrorInfo --
  *
  * Add information to the error info stack about index that just failed.
@@ -505,8 +551,7 @@ AddLibIndexErrorInfo (interp, indexName)
 }
 
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * ProcessIndexFile --
  *
  * Open and process a package library index file (.tndx).  Creates entries
@@ -578,8 +623,9 @@ ProcessIndexFile (interp, tlibFilePath, tndxFilePath)
     }
 
   reachedEOF:
-    Tcl_Close (indexChannel);
-    Tcl_DStringFree (&lineBuffer);
+    Tcl_DStringFree (&lineBuffer);    
+    if (Tcl_Close (NULL, indexChannel) != TCL_OK)
+        goto fileError;
 
     return TCL_OK;
 
@@ -608,12 +654,11 @@ ProcessIndexFile (interp, tlibFilePath, tndxFilePath)
         ckfree ((char *) lineArgv);
     Tcl_DStringFree (&lineBuffer);
     if (indexChannel != NULL)
-        Tcl_Close (indexChannel);
+        Tcl_Close (NULL, indexChannel);
     return TCL_ERROR;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * BuildPackageIndex --
  *
  * Call the "buildpackageindex" Tcl procedure to rebuild a package index.
@@ -636,7 +681,7 @@ BuildPackageIndex (interp, tlibFilePath)
 
     Tcl_DStringInit (&command);
 
-    Tcl_DStringAppend (&command, "source $tclx_library/buildidx.tcl;", -1);
+    Tcl_DStringAppend (&command, "source [file join $tclx_library buildidx.tcl];", -1);
     Tcl_DStringAppend (&command, "buildpackageindex ", -1);
     Tcl_DStringAppend (&command, tlibFilePath, -1);
 
@@ -650,8 +695,7 @@ BuildPackageIndex (interp, tlibFilePath)
     return result;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * LoadPackageIndex --
  *
  * Load a package .tndx file.  Rebuild .tlib if non-existant or out of
@@ -660,25 +704,33 @@ BuildPackageIndex (interp, tlibFilePath)
  * Parameters
  *   o interp (I) - A pointer to the interpreter, error returned in result.
  *   o tlibFilePath (I) - Absolute path name to the library file.
+ *   o indexNameClass (I) - TCLLIB_TNDX if the index file should the suffix
+ *     ".tndx" or TCLLIB_TND if it should have ".tnd".
  * Returns:
  *   TCL_OK or TCL_ERROR.
  *-----------------------------------------------------------------------------
  */
 static int
-LoadPackageIndex (interp, tlibFilePath)
-    Tcl_Interp *interp;
-    char       *tlibFilePath;
+LoadPackageIndex (interp, tlibFilePath, indexNameClass)
+    Tcl_Interp       *interp;
+    char             *tlibFilePath;
+    indexNameClass_t  indexNameClass;
 {
-    Tcl_DString  tndxFilePath;
+    Tcl_DString tndxFilePath;
+
     struct stat  tlibStat;
     struct stat  tndxStat;
 
     Tcl_DStringInit (&tndxFilePath);
 
+    /*
+     * Modify library file path to be the index file path.
+     */
     Tcl_DStringAppend (&tndxFilePath, tlibFilePath, -1);
     tndxFilePath.string [tndxFilePath.length - 3] = 'n';
     tndxFilePath.string [tndxFilePath.length - 2] = 'd';
-    tndxFilePath.string [tndxFilePath.length - 1] = 'x';
+    if (indexNameClass == TCLLIB_TNDX)
+        tndxFilePath.string [tndxFilePath.length - 1] = 'x';
 
     /*
      * Get library's modification time.  If the file can't be accessed, set
@@ -710,8 +762,7 @@ LoadPackageIndex (interp, tlibFilePath)
     return TCL_ERROR;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * LoadOusterIndex --
  *
  * Load a standard Tcl index (tclIndex).  A special proc is used so that the
@@ -770,8 +821,99 @@ LoadOusterIndex (interp, indexFilePath)
     return TCL_ERROR;
 }
 
-/*
+/*-----------------------------------------------------------------------------
+ * LoadDirIndexCallback --
+ *
+ *   Function called for every directory entry for LoadDirIndexes.
+ *
+ * Parameters
+ *   o interp (I) - Interp is passed though.
+ *   o path (I) - Normalized path to directory.
+ *   o fileName (I) - Tcl normalized file name in directory.
+ *   o caseSensitive (I) - Are the file names case sensitive?  Always
+ *     TRUE on Unix.
+ *   o clientData (I) - Pointer to a boolean that is set to TRUE if an error
+ *     occures while porocessing the index file.
+ * Returns:
+ *   TCL_OK or TCL_ERROR.
  *-----------------------------------------------------------------------------
+ */
+static int
+LoadDirIndexCallback (interp, path, fileName, caseSensitive, clientData)
+    Tcl_Interp  *interp;
+    char        *path;
+    char        *fileName;
+    int          caseSensitive;
+    ClientData   clientData;
+{
+    int *indexErrorPtr = (int *) clientData;
+    int nameLen;
+    char *chkName;
+    indexNameClass_t indexNameClass;
+    Tcl_DString chkNameBuf, filePath;
+
+    /*
+     * If the volume not case sensitive, convert the name to lower case.
+     */
+    Tcl_DStringInit (&chkNameBuf);
+    chkName = fileName;
+    if (!caseSensitive) {
+        chkName = Tcl_DStringAppend (&chkNameBuf, fileName, -1);
+        Tcl_DownShift (chkName, chkName);
+    }
+
+    /*
+     * Figure out if its an index file.
+     */
+    nameLen = strlen (chkName);
+    if ((nameLen > 5) && STREQU (chkName + nameLen - 5, ".tlib")) {
+        indexNameClass = TCLLIB_TNDX;
+    } else if ((nameLen > 4) && STREQU (chkName + nameLen - 4, ".tli")) {
+        indexNameClass = TCLLIB_TND;
+    } else if ((caseSensitive && STREQU (chkName, "tclIndex")) ||
+               ((!caseSensitive) && STREQU (chkName, "tclindex"))) {
+        indexNameClass = TCLLIB_OUSTER;
+    } else {
+        Tcl_DStringFree (&chkNameBuf);
+        return TCL_OK;  /* Not an index, skip */
+    }
+    Tcl_DStringFree (&chkNameBuf);
+
+    /*
+     * Assemble full path to library file.
+     */
+    Tcl_DStringInit (&filePath);
+    TclX_JoinPath (path, fileName, &filePath);
+
+    /*
+     * Skip index it can't be accessed.
+     */
+    if (access (filePath.string, R_OK) < 0)
+        goto exitPoint;
+
+    /*
+     * Process the index according to its type.
+     */
+    if (indexNameClass == TCLLIB_OUSTER) {
+        if (LoadOusterIndex (interp, filePath.string) != TCL_OK)
+            goto errorExit;
+    } else {
+        if (LoadPackageIndex (interp, filePath.string,
+                              indexNameClass) != TCL_OK)
+            goto errorExit;
+    }
+
+  exitPoint:
+    Tcl_DStringFree (&filePath);
+    return TCL_OK;
+
+  errorExit:
+    Tcl_DStringFree (&filePath);
+    *indexErrorPtr = TRUE;
+    return TCL_ERROR;
+}
+
+/*-----------------------------------------------------------------------------
  * LoadDirIndexes --
  *
  *     Load the indexes for all package library (.tlib) or a Ousterhout
@@ -789,68 +931,27 @@ LoadDirIndexes (interp, dirName)
     Tcl_Interp  *interp;
     char        *dirName;
 {
-    DIR           *dirPtr;
-    struct dirent *entryPtr;
-    int            dirNameLen, nameLen;
-    Tcl_DString    filePath;
+    int indexError = FALSE;
 
-    dirPtr = opendir (dirName);
-    if (dirPtr == NULL)
-        return TCL_OK;   /* Skip directory */
-
-    Tcl_DStringInit (&filePath);
-    Tcl_DStringAppend (&filePath, dirName, -1);
-    Tcl_DStringAppend (&filePath, "/",     -1);
-
-    dirNameLen = strlen (dirName) + 1;  /* Include `/' */
-
-    while (TRUE) {
-        entryPtr = readdir (dirPtr);
-        if (entryPtr == NULL)
-            break;
-        nameLen = strlen (entryPtr->d_name);
-
-        if ((nameLen > 5) && 
-            ((STREQU (entryPtr->d_name + nameLen - 5, ".tlib")) ||
-             (STREQU (entryPtr->d_name, "tclIndex")))) {
-
-            /*
-             * Append the file name on to the directory.
-             */
-            Tcl_DStringTrunc (&filePath, dirNameLen);
-            Tcl_DStringAppend (&filePath, entryPtr->d_name, -1);
-
-            /*
-             * Skip index it can't be accessed.
-             */
-            if (access (filePath.string, R_OK) < 0)
-                continue;
-
-            /*
-             * Process the index according to its type.
-             */
-            if (entryPtr->d_name [nameLen - 5] == '.') {
-                if (LoadPackageIndex (interp, filePath.string) != TCL_OK)
-                    goto errorExit;
-            } else {
-                if (LoadOusterIndex (interp, filePath.string) != TCL_OK)
-                    goto errorExit;
-            }
+    /*
+     * This is a little tricky.  We want to skip directories we can't read,
+     * read, but if we get an error processing an index, we want
+     * to report it.  A boolean is passed in to indicate if the error
+     * returned involved parsing the file.
+     */
+    if (TclXOSWalkDir (interp, dirName, FALSE, /* hidden */
+                       LoadDirIndexCallback,
+                       (ClientData) &indexError) == TCL_ERROR) {
+        if (!indexError) {
+            Tcl_ResetResult (interp);
+            return TCL_OK;
         }
+        return TCL_ERROR;
     }
-
-    Tcl_DStringFree (&filePath);
-    closedir (dirPtr);
     return TCL_OK;
-
-  errorExit:
-    Tcl_DStringFree (&filePath);
-    closedir (dirPtr);
-    return TCL_ERROR;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * LoadPackageIndexes --
  *
  * Loads the all indexes for all package libraries (.tlib) or a Ousterhout
@@ -909,8 +1010,7 @@ LoadPackageIndexes (interp, infoPtr, path)
     return result;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * Tcl_Auto_load_pkgCmd --
  *
  *   Implements the command:
@@ -947,8 +1047,7 @@ Tcl_Auto_load_pkgCmd (dummy, interp, argc, argv)
     return result;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * AddInProgress --
  *
  *   An a command to the table of in progress commands.  If the command is
@@ -980,8 +1079,7 @@ AddInProgress (interp, infoPtr, command)
     return TCL_OK;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * RemoveInProgress --
  *
  *   Remove a command from the in progress table.
@@ -1007,8 +1105,7 @@ RemoveInProgress (interp, infoPtr, command)
     Tcl_DeleteHashEntry (entryPtr);
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * LoadChangedPathPackageIndexes --
  *
  *   Determine what part of the path has changed and load the package indexes
@@ -1064,8 +1161,7 @@ LoadChangedPathPackageIndexes (interp, infoPtr, oldPath, path)
 
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * LoadAutoPath --
  *
  *   Load all indexs on the auto_path variable.  If auto_path has not changed
@@ -1118,11 +1214,128 @@ LoadAutoPath (interp, infoPtr)
     return TCL_OK;
 }
 
+/*-----------------------------------------------------------------------------
+ * LoadStdProc --
+ *
+ *   Check the "auto_index" array for code to load a command and eval it.
+ *
+ * Parameters
+ *   o interp (I) - A pointer to the interpreter, error returned in result.
+ *   o command (I) - The command to load.
+ * Returns:
+ *   TCL_OK if the command was loaded.
+ *   TCL_CONTINUE if the command is not in the index.
+ *   TCL_ERROR if an error occured.
+ *-----------------------------------------------------------------------------
+ */
+static int
+LoadStdProc (interp, command)
+    Tcl_Interp  *interp;
+    char        *command;
+{
+    char *loadCmd;
+
+    loadCmd = Tcl_GetVar2 (interp, AUTO_INDEX, command, TCL_GLOBAL_ONLY);
+    if (loadCmd == NULL)
+        return TCL_CONTINUE;   /* Not found */
+
+    if (Tcl_GlobalEval (interp, loadCmd) == TCL_ERROR)
+        return TCL_ERROR;
+    return TCL_OK;
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ * LoadITclImportProc --
+ *
+ *   Check if the specified command is an auto load ITcl imported proc.  If so,
+ * evaluated the load command it.  Don't call if namespaces are not available.
+ *
+ * Parameters
+ *   o interp (I) - A pointer to the interpreter, error returned in result.
+ *   o command (I) - The command to load.
+ * Returns:
+ *   TCL_OK if the command was loaded.
+ *   TCL_CONTINUE if the command was not found.
+ *   TCL_ERROR if an error occured.
+ *-----------------------------------------------------------------------------
+ */
+static int
+LoadITclImportProc (interp, command)
+    Tcl_Interp  *interp;
+    char        *command;
+{
+    static char importCmd [] = "import all";
+    Tcl_DString tmpResult, fullName;
+    char **nsSearchPathv;
+    int nsSearchPathc, idx, nameSpaceLen;
+    char *nameSpace, *nextPtr, *loadCmd;
+
+    Tcl_DStringInit (&tmpResult);
+    Tcl_DStringInit (&fullName);
+
+    /*
+     * Get the namespace search path.
+     */
+    if (Tcl_Eval (interp, importCmd) == TCL_ERROR)
+        goto errorExit;
+    Tcl_DStringAppend (&tmpResult, interp->result, -1);
+    
+    if (Tcl_SplitList (interp, tmpResult.string,
+                       &nsSearchPathc,
+                       &nsSearchPathv) == TCL_ERROR)
+        goto errorExit;
+
+    /*
+     * Search the path for the name.
+     */
+    for (idx = 0; idx < nsSearchPathc; idx++) {
+        Tcl_DStringSetLength (&fullName, 0);
+        if (TclFindElement (interp,
+                            nsSearchPathv [idx],
+                            &nameSpace,
+                            &nextPtr,
+                            &nameSpaceLen,
+                            NULL))
+            goto errorExit;
+        nameSpace [nameSpaceLen] = '\0';
+        if (!STREQU (nameSpace, "::")) {
+            Tcl_DStringAppend (&fullName, nameSpace, -1);
+        }
+        Tcl_DStringAppend (&fullName, "::", -1);
+        Tcl_DStringAppend (&fullName, command, -1);
+        loadCmd = Tcl_GetVar2 (interp, AUTO_INDEX,
+                               fullName.string, TCL_GLOBAL_ONLY);
+        if (loadCmd != NULL)
+            break;
+    }
+
+    if (loadCmd == NULL) {
+        Tcl_DStringFree (&tmpResult);
+        Tcl_DStringFree (&fullName);
+        return TCL_CONTINUE;   /* Not found */
+    }
+
+    if (Tcl_GlobalEval (interp, loadCmd) == TCL_ERROR)
+        goto errorExit;
+
+    Tcl_DStringFree (&tmpResult);
+    Tcl_DStringFree (&fullName);
+    return TCL_OK;
+
+    
+  errorExit:
+    Tcl_DStringFree (&tmpResult);
+    Tcl_DStringFree (&fullName);
+    return TCL_ERROR;
+}
+
 /*
  *-----------------------------------------------------------------------------
  * LoadCommand --
  *
- *   Check the "auto_index" array for code to load a command and eval it.
+ *   Check the "auto_index" array for code to load a command and eval it.  Also
+ * check ITcl import list.
  *
  * Parameters
  *   o interp (I) - A pointer to the interpreter, error returned in result.
@@ -1138,15 +1351,17 @@ LoadCommand (interp, command)
     Tcl_Interp  *interp;
     char        *command;
 {
-    char        *loadCmd;
-    Tcl_CmdInfo  cmdInfo;
+    int result;
+    Tcl_CmdInfo cmdInfo;
 
-    loadCmd = Tcl_GetVar2 (interp, AUTO_INDEX, command, TCL_GLOBAL_ONLY);
-    if (loadCmd == NULL)
-        return TCL_CONTINUE;   /* Not found */
-
-    if (Tcl_GlobalEval (interp, loadCmd) == TCL_ERROR)
+    result = LoadStdProc (interp, command);
+    if (haveNameSpaces && (result == TCL_CONTINUE))
+        result = LoadITclImportProc (interp, command);
+    if (result == TCL_CONTINUE)
+        return TCL_CONTINUE;
+    if (result == TCL_ERROR)
         return TCL_ERROR;
+
     Tcl_ResetResult (interp);
 
     if (Tcl_GetCommandInfo (interp, command, &cmdInfo))
@@ -1158,8 +1373,7 @@ LoadCommand (interp, command)
     return TCL_ERROR;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * Tcl_LoadlibindexCmd --
  *
  *   This procedure is invoked to process the "Loadlibindex" Tcl command:
@@ -1200,32 +1414,34 @@ Tcl_LoadlibindexCmd (dummy, interp, argc, argv)
      */
     pathLen = strlen (pathName);
 
-    if ((pathLen > 5) && (pathName [pathLen - 5] == '.')) {
-        if (!STREQU (pathName + pathLen - 5, ".tlib"))
-            goto invalidName;
-        if (LoadPackageIndex (interp, pathName) != TCL_OK)
+    if ((pathLen > 5) && STREQU (pathName + pathLen - 5, ".tlib")) {
+        if (LoadPackageIndex (interp, pathName, TCLLIB_TNDX) != TCL_OK)
             goto errorExit;
-    } else {
-        if (!STREQU (pathName + pathLen - 9, "/tclIndex"))
-            goto invalidName;
+    } else if ((pathLen > 4) && STREQU (pathName + pathLen - 4, ".tli")) {
+        if (LoadPackageIndex (interp, pathName, TCLLIB_TND) != TCL_OK)
+            goto errorExit;
+    } else if (STREQU (pathName + pathLen - 9, "/tclIndex") ||
+               STREQU (pathName + pathLen - 9, "/tclindex")) {
+        /* Should we check for DOS before allowing lower case???*/
         if (LoadOusterIndex (interp, pathName) != TCL_OK)
             goto errorExit;
+    } else {
+        Tcl_AppendResult (interp, "invalid library name, must have an ",
+                          " extension of \".tlib\", \".tli\" or the name ",
+                          "\"tclIndex\", got \"", argv [1], "\"",
+                          (char *) NULL);
+        goto errorExit;
     }
+
     Tcl_DStringFree (&pathNameBuf);
     return TCL_OK;
 
-  invalidName:
-    Tcl_AppendResult (interp, "invalid library name, must have an extension ",
-                      "of \".tlib\" or the name \"tclIndex\", got \"",
-                      argv [1], "\"", (char *) NULL);
-
   errorExit:
     Tcl_DStringFree (&pathNameBuf);
     return TCL_ERROR;;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * Tcl_auto_loadCmd --
  *
  *   This procedure is invoked to process the "auto_load" Tcl command:
@@ -1312,8 +1528,7 @@ Tcl_Auto_loadCmd (clientData, interp, argc, argv)
     return TCL_ERROR;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * TclLibCleanUp --
  *
  *   Release the client data area when the interpreter is deleted.
@@ -1339,8 +1554,7 @@ TclLibCleanUp (clientData, interp)
     ckfree ((char *) infoPtr);
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * TclX_LibraryInit --
  *
  *   Initialize the Extended Tcl library facility commands.
@@ -1351,6 +1565,7 @@ TclX_LibraryInit (interp)
     Tcl_Interp *interp;
 {
     libInfo_t *infoPtr;
+    Tcl_CmdInfo cmdInfo;
 
     infoPtr = (libInfo_t *) ckalloc (sizeof (libInfo_t));
 
@@ -1360,11 +1575,18 @@ TclX_LibraryInit (interp)
     Tcl_CallWhenDeleted (interp, TclLibCleanUp, (ClientData) infoPtr);
 
     Tcl_CreateCommand (interp, "auto_load_pkg", Tcl_Auto_load_pkgCmd,
-                      (ClientData) infoPtr, (void (*)()) NULL);
+                      (ClientData) infoPtr, (Tcl_CmdDeleteProc*) NULL);
     Tcl_CreateCommand (interp, "auto_load", Tcl_Auto_loadCmd,
-                      (ClientData) infoPtr, (void (*)()) NULL);
+                      (ClientData) infoPtr, (Tcl_CmdDeleteProc*) NULL);
     Tcl_CreateCommand (interp, "loadlibindex", Tcl_LoadlibindexCmd,
-                      (ClientData) infoPtr, (void (*)()) NULL);
+                      (ClientData) infoPtr, (Tcl_CmdDeleteProc*) NULL);
+
+    /*
+     * Check for ITcl namespaces.
+     */
+    haveNameSpaces = Tcl_GetCommandInfo (interp, 
+                                         nameSpaceCheckCommand,
+                                         &cmdInfo);
     return TCL_OK;
 }
 
