@@ -13,7 +13,7 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: tclhelp.tcl,v 5.3 1996/02/12 18:17:23 markd Exp $
+# $Id: tclhelp.tcl,v 8.0 1996/11/21 00:25:33 markd Exp $
 #------------------------------------------------------------------------------
 
 #------------------------------------------------------------------------------
@@ -414,6 +414,6 @@ proc tkhelp addPaths {
 if [catch {
     tkhelp $argv
 } msg] {
-    tkerror $msg
+    bgerror $msg
 }
 
